@@ -1,6 +1,9 @@
 from ZTUtils import make_query
 
 from zope.interface import implements
+from zope.interface import providedBy
+from Products.ATContentTypes.interface import IATTopic
+
 from zope import schema
 from zope.formlib import form
 from zope.component import getMultiAdapter
@@ -18,6 +21,8 @@ from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 
 from collective.portlet.relateditems import RelatedItemsMessageFactory as _
 
+# TODO: Folderish get categories from children
+
 DEFAULT_ALLOWED_TYPES = (
     'News Item',
     'Document',
@@ -38,7 +43,7 @@ class IRelatedItems(IPortletDataProvider):
         title=_(u'Number of related items to display'),
         description=_(u'How many related items to list.'),
         required=True,
-        default=5
+        default=10
     )
 
     states = schema.Tuple(
@@ -139,6 +144,67 @@ class Renderer(base.Renderer):
             req_items['portal_type'] = list(self.data.allowed_types)
         return '%s/search?%s' % (portal_url, make_query(req_items))
 
+    
+    def _contents(self):
+        contents = []
+        # Collection
+        if IATTopic.providedBy(self.context):
+            contents = self.context.queryCatalog(sort_limit=5)
+        # probably a folder
+        else:
+            contents = self.context.getFolderContents()
+              
+        return contents    
+ 
+    def _itemQuery(self, value):
+        search_query = []
+
+        # Include categories in the search query
+        # Transformed to lowercase to not disrupt search (AND,OR,etc)
+        keywords = map(lambda x: x.lower(), value.Subject())
+
+        # Include words from title in the search query
+        # Transformed to lowercase to not disrupt search (AND,OR,etc) 
+        title = value.Title().lower().split()
+        
+        search_query = title + keywords
+        return search_query
+
+    def _itemsQuery(self, values):
+        query = ''
+        items = []
+        for item in values:
+            items += self._itemQuery(item)
+        
+        # remove duplicated search keywords
+        items = self.uniq(items)
+                  
+        query = items.pop(0)
+        for item in items:
+            query += ' OR ' + item
+                        
+        return query
+
+    def uniq(self, alist):    # Fastest order preserving
+        set = {}
+        return [set.setdefault(e,e) for e in alist if e not in set]
+
+    def _query(self):    
+        keywords = []        
+        context = aq_inner(self.context)
+        
+        # TODO: test if a collection limited to 3 items show anything...
+        # get items in folder or collection
+        contents = []
+        if self.context.isPrincipiaFolderish:
+            contents = self._contents()
+            contents = map(lambda x: x.getObject(), contents)     
+                
+        contents.insert(0,self.context);
+        
+        search_query = self._itemsQuery(contents)
+        return search_query
+            
     @memoize
     def _data(self):
         plone_tools = getMultiAdapter((self.context, self.request),
@@ -146,20 +212,34 @@ class Renderer(base.Renderer):
         context = aq_inner(self.context)
         keywords = context.Subject()
         here_path = ('/').join(context.getPhysicalPath())
+        
+        # Exclude items from related if contained in folderish
+        content = []
+        if self.context.isPrincipiaFolderish:
+            content = self._contents()
+        exclude_items = map(lambda x: x.getPath(), content)
+              
+        exclude_items += [here_path]
+        #print exclude_items
+        #exclude_items = [here_path]       
+        
         catalog = plone_tools.catalog()
         limit = self.data.count
         # increase by one since we'll get the current item
-        extra_limit = limit + 1
-        #print keywords
+        extra_limit = limit + len(exclude_items)
+        
+        search_query = self._query()
+        #print 'search_query = '+search_query  
+                
         results = catalog(portal_type=self.data.allowed_types,
-                          Subject=keywords,
-                          SearchableText='marnix OR presentation',
+                          #Subject=keywords,
+                          SearchableText= search_query,
                           review_state=self.data.states,
-                          sort_on='Date',
-                          sort_order='reverse',
+                          #sort_on='Date',
+                          #sort_order='reverse',
                           sort_limit=extra_limit)
         # filter out the current item
-        self.all_results = [res for res in results if res.getPath() != here_path]
+        self.all_results = [res for res in results if not res.getPath() in exclude_items]
         return self.all_results[:limit]
 
 class AddForm(base.AddForm):
